// Fichier Propulsion.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <i2c.h>			// Librairie pour la liaison srie

#include "I2C_Master.h"	





// --------------------------------------------------------------------------
/* Initialise les broches du bus I2C en sortie
 * Configure la liaison I2c  en maitre, 7bits, 100Kbits/s
 * Pas de paramtre
*/
void I2C_Init(void)
{
	TRIS_SDA=0;		// SDA en sortie
	TRIS_SCL=0;		// SCL en sortie

	OpenI2C(MASTER,SLEW_OFF);	// En matre, vitesse normale
	SSPADD = 48;				// 48: 100Khz avec un quartz de 20MHz
}	


// --------------------------------------------------------------------------
/* Lit une donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 * 
 * Paramtre retourn : 
 *   Valeur lue
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/

int I2C_Receive(unsigned char Addr)
{
	unsigned char Data;				
	INTCONbits.TMR0IE=0;		// Interdit l'interruption (pour viter de couper une trame au milieu)
    IdleI2C();					// Attends que le module MSSP soit disponible
    StartI2C();					// Envoie du START
    IdleI2C();					// Attends que le module MSSP soit disponible (Fin de l'envoie du START)
 
 	if (WriteI2C (Addr | 0x01))	// Envoie l'adresse en prcisant mode rception
 	{
       	StopI2C();				// Pas de rponse, on envoie un STOP
		INTCONbits.TMR0IE=1;	// Rautorise l'interruption
       	return -1;           	// Pas d'acknoledgement ou une collision : on retourne -1
    }        
       	
    Data = ReadI2C();			// Rcupre l'octet et le mmorise dans le tableau
    NotAckI2C(); 				// On envoie un NAk ..
	IdleI2C();					// Attends que le module MSSP soit disponible (Fin de l'envoie du Ack ou du NAck)
			
	StopI2C();					// Fin de la transmission, on envoie un STOP
    IdleI2C();					// Attends que le module MSSP soit disponible (Fin de l'envoie du STOP)
	INTCONbits.TMR0IE=1;		// Rautorise l'interruption
    return Data;				// Tout s'est bien pass
}



// --------------------------------------------------------------------------
/* Lit des donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 *  - Le second paramtre est le nombre de donnes  lire [0:255]
 *  - Le dernier paramtre est un tableau contenant les donnes lues
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/

int I2C_Receive_x(unsigned char Addr, unsigned char NbData, unsigned char *Data)
{
			
	INTCONbits.TMR0IE=0;		// Interdit l'interruption (pour viter de couper une trame au milieu)	
    IdleI2C();					// Attends que le module MSSP soit disponible
    StartI2C();					// Envoie du START
    IdleI2C();					// Attends que le module MSSP soit disponible (Fin de l'envoie du START)
 
 	if (WriteI2C (Addr | 0x01))	// Envoie l'adresse en prcisant mode rception
 	{
       	StopI2C();				// Pas de rponse, on envoie un STOP
		INTCONbits.TMR0IE=1;	// Rautorise l'interruption
       	return -1;           	// Pas d'acknoledgement ou une collision : on retourne -1
    }        
     
    
	for (;NbData>0;NbData--)	// Pour toute les donnes  transmettre
	{	
       	*Data = ReadI2C();		// Rcupre l'octet et le mmorise dans le tableau
       	Data++;					// Passe  la donne suivante
		if (NbData==1)  		// Si c'est la dernire donne ..
			NotAckI2C(); 		// .. en envoie un NAk ..
		else 					//
			AckI2C(); 			// .. sinon, on envoie un Ack
		IdleI2C();				// Attends que le module MSSP soit disponible (Fin de l'envoie du Ack ou du NAck)		
	}
		
	StopI2C();					// Fin de la transmission, on envoie un STOP
    IdleI2C();					// Attends que le module MSSP soit disponible (Fin de l'envoie du STOP)
	INTCONbits.TMR0IE=1;	// Rautorise l'interruption
    return 0;					// Tout s'est bien pass
}





// --------------------------------------------------------------------------
/* Envoie une donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 *  - Le second paramtre est la donne  envoyer
 * 
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/

int I2C_Send(unsigned char Addr, unsigned char Data)
{
	//	INTCONbits.TMR0IE=0;		// Interdit l'interruption (pour viter de couper une trame au milieu)	
        IdleI2C();					// Attends que la ligne soit au repos	
        StartI2C();					// Envoie du START
        IdleI2C();					// Attends que la ligne soit au repos	
        
        if (WriteI2C (Addr & 0xFE))	// Envoie l'adresse en prcisant mode transmission
 		{
			IdleI2C();					// Attends que la ligne soit au repos	
       		StopI2C();				// Pas de rponse, on envoie un STOP
	//		INTCONbits.TMR0IE=1;	// Rautorise l'interruption
       		return -1;           	// Pas d'acknoledgement ou une collision : on retourne -1
    	}   
       	
		if (WriteI2C (Data))		// Envoie la donne
 		{
	 		IdleI2C();					// Attends que la ligne soit au repos	
     
       		StopI2C();				// Pas de rponse, on envoie un STOP
	//	INTCONbits.TMR0IE=1;	// Rautorise l'interruption
       		return -2;           	// Pas d'acknoledgement ou une collision : on retourne -2
    	}   
    	IdleI2C();
        StopI2C();					// Fin de la transmission, on envoie un STOP
	//	INTCONbits.TMR0IE=1;		// Rautorise l'interruption
        return 0;					// Tout s'est bien pass
}



// --------------------------------------------------------------------------
/* Envoie des donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 *  - Le second paramtre est le nombre de donnes  transmettre [0:255]
 *  - Le dernier paramtre est un tableau contenant les donnes  envoyer
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/

int I2C_Send_x(unsigned char Addr, unsigned char NbData, const unsigned char *Data)
{
		INTCONbits.TMR0IE=0;		// Interdit l'interruption (pour viter de couper une trame au milieu)	
        IdleI2C();					// Attends que la ligne soit au repos	
        StartI2C();					// Envoie du START
        IdleI2C();					// Attends que la ligne soit au repos	
        
        if (WriteI2C (Addr & 0xFE))	// Envoie l'adresse en prcisant mode transmission
 		{
       		StopI2C();				// Pas de rponse, on envoie un STOP
			INTCONbits.TMR0IE=1;	// Rautorise l'interruption
       		return -1;           	// Pas d'acknoledgement ou une collision : on retourne -1
    	}   
       	
		for (;NbData>0;NbData--)	// Pour toute les donnes  transmettre
		{
			
			if (WriteI2C (*Data))	// Envoie la donne
 			{
       			StopI2C();				// Pas de rponse, on envoie un STOP
				INTCONbits.TMR0IE=1;	// Rautorise l'interruption
       			return -1;           	// Pas d'acknoledgement ou une collision : on retourne -2
    		}   
    		Data++;						// Passe  la donne suivante
   		}
        StopI2C();					// Fin de la transmission, on envoie un STOP
		INTCONbits.TMR0IE=1;		// Rautorise l'interruption
        return 0;					// Tout s'est bien pass
}